@@ -1,6 +1,6 @@
 package DBM::Deep::Engine::DBI;
 
-use 5.008_000;
+use 5.006_000;
 
 use strict;
 use warnings FATAL => 'all';
@@ -23,7 +23,6 @@ sub new {
 
     my $self = bless {
         storage => undef,
-        external_refs => undef,
     }, $class;
 
     # Grab the parameters we want to use
@@ -217,9 +216,7 @@ sub write_value {
         or die "Cannot load sector at '@{[$obj->_base_offset]}'\n";;
 
     my ($type, $class);
-    if (
-     $r eq 'ARRAY' || $r eq 'HASH' and ref $value ne 'DBM::Deep::Null'
-    ) {
+    if ( $r eq 'ARRAY' || $r eq 'HASH' ) {
         my $tmpvar;
         if ( $r eq 'ARRAY' ) {
             $tmpvar = tied @$value;
@@ -264,13 +261,6 @@ sub write_value {
             DBM::Deep->_throw_error( "Cannot store something that is tied." );
         }
 
-        if ( ref $value eq 'DBM::Deep::Null' ) {
-            DBM::Deep::_warnif(
-             'uninitialized', 'Assignment of stale reference'
-            );
-            $value = undef;
-        }
-
         $class = 'DBM::Deep::Sector::DBI::Scalar';
         $type  = 'S';
     }
@@ -357,10 +347,6 @@ sub supports {
     return;
 }
 
-sub db_version {
-    return '1.0020'
-}
-
 sub clear {
     my $self = shift;
     my $obj = shift;
