@@ -1,5 +1,3 @@
-# This was copied from MARCEL's Class::Null. However, I couldn't use it because
-# I need an undef value, not an implementation of the Null Class pattern.
 package DBM::Deep::Null;
 
 use 5.006_000;
@@ -13,8 +11,8 @@ DBM::Deep::Null
 
 =head1 PURPOSE
 
-This is an internal-use-only object for L<DBM::Deep/>. It acts as a NULL object
-in the same vein as MARCEL's L<Class::Null/>. I couldn't use L<Class::Null/>
+This is an internal-use-only object for L<DBM::Deep>. It acts as a NULL object
+in the same vein as MARCEL's L<Class::Null>. I couldn't use L<Class::Null>
 because DBM::Deep needed an object that always evaluated as undef, not an
 implementation of the Null Class pattern.
 
@@ -27,7 +25,7 @@ It is used to represent null sectors in DBM::Deep.
 use overload
     'bool'   => sub { undef },
     '""'     => sub { undef },
-    '0+'     => sub { undef },
+    '0+'     => sub { 0 },
     fallback => 1,
     nomethod => 'AUTOLOAD';
 
